@@ -14,6 +14,8 @@ class JobType(Enum):
     Class to contain allowed Gaussian job types. Not an exhaustive list, but should be fairly comprehensive.
 
     The value should be the Gaussian keyword, to permit automatic assignment.
+
+    All jobs have type ``SP`` by default.
     """
 
     SP = "sp"
@@ -24,49 +26,45 @@ class JobType(Enum):
     POP = "pop"
     FORCE = "force"
 
+#### This static variable tells what properties are expected from each JobType.
+EXPECTED_PROPERTIES = {
+    "sp": ["energy", "scf_iterations",],
+    "opt": ["rms_displacement", "rms_force", ],
+    "freq": ["gibbs_free_energy", "enthalpy", "frequencies",],
+    "nmr": ["isotropic_shielding",],
+    "pop": [],
+    "force": ["forces",],
+}
+
 
 class GaussianFile(File):
     """
     Class for Gaussian files. Composes ``ConformationalEnsemble``.
 
     Attributes:
-        molecules (Ensemble): ``ConformationalEnsemble`` instance
+        ensemble (Ensemble): ``ConformationalEnsemble`` instance
         job_types (list): list of `job_type` instances
         route_card (str): optional, route card of .gjf file
         link0 (dict): optional, dictionary of Link 0 commands (e.g. {"mem": "32GB", "nprocshared": 16})
         footer (str): optional, footer of .gjf file
         success (int): number of successful terminations (should be 1 for an opt, 2 for opt and then freq, 1 for a single point energy, etc)
-        energies (list): list of energies for each cycle
-        scf_iterations (list): number of iterations per cycle
-        max_displacements (list): list of max displacement values for each cycle
-        rms_displacements (list): list of rms displacement values for each cycle
-        max_forces (list): list of max force values for each cycle
-        rms_forces (list): list of rms force values for each cycle
-        gradients (list): list of gradient values for each cycle
-        frequencies (list): list of frequencies
-        gibbs_free_energy (float): gibbs free energy, from vibrational correction
-        enthalpy (float): enthalpy, from vibrational correction
         title (str): optional, title of .gjf file
     """
 
     def __init__(
-        self, atomic_numbers, geometries, bonds=None, charge=None, multiplicity=None, job_types=None, route_card=None, link0=None, footer=None, title="title",
+        self, job_types, route_card=None, link0=None, footer=None, title="title", success=0
     ):
         """
         Create new GaussianFile object.
 
-		Args:
-            atomic_numbers (list): list of atomic numbers
-            geometries (list): list of lists of 3-tuples of xyz coordinates
-            bonds (nx.Graph): Graph object containing connectivity information (1-indexed)
-            charges (int): the charge of the molecules
-            multiplicities (int): the spin states of the molecules (1 corresponds to singlet, 2 to doublet, 3 to triplet, etc. -- so a multiplicity of 1 is equivalent to S=0)
+	Args:
             job_types (list): list of ``job_type`` instances
             route_card (str): optional, route card of ``.gjf`` file
             link0 (dict): optional, Link 0 commands of ``.gjf`` file
             footer (str): optional, footer of ``.gjf`` file
             title (str): optional, title of ``.gjf`` file
-		"""
+            success (int): num successful terminations
+	"""
 
         if route_card and not isinstance(route_card, str):
             raise TypeError("route card needs to be a string")
@@ -80,19 +78,23 @@ class GaussianFile(File):
         if title and not isinstance(title, str):
             raise TypeError("title needs to be a string")
 
+        if success and not isinstance(success, int):
+            raise TypeError("success needs to be an integer")
+
         if job_types is not None:
             if not all(isinstance(job, JobType) for job in job_types):
                 raise TypeError(f"invalid job type {job}")
 
-        if (len(atomic_numbers) > 0) and (len(geometries) > 0):
-            arguments = {"atomic_numbers": atomic_numbers, "geometries": geometries, "bonds": bonds, "charge": charge, "multiplicity": multiplicity}
-            self.molecules = ConformationalEnsemble(**arguments)
-
+        self.ensemble = ConformationalEnsemble()
         self.route_card = route_card
         self.link0 = link0
         self.footer = footer
         self.title = title
         self.job_types = job_types
+        self.success = success
+
+    def __str__(self):
+        return f"GaussianFile (title=\"{str(self.title)}\", {len(self.ensemble)} entries in Ensemble)"
 
     @classmethod
     def write_molecule_to_file(cls, filename, molecule, route_card, link0={"mem": "32GB", "nprocshared": 16}, footer=None, title="title", append=False, print_symbol=False):
@@ -115,6 +117,8 @@ class GaussianFile(File):
         if (route_card is None) or (not isinstance(route_card, str)):
             raise ValueError("can't write a file without a route card")
 
+        assert re.match(r"^#p", route_card), f"route card doesn't start with #p: {route_card}"
+
         #### generate the text
         text = ""
         if append:
@@ -131,7 +135,9 @@ class GaussianFile(File):
             line = molecule.get_vector(index)
             if print_symbol:
                 Z = get_symbol(Z)
-            text += f"{Z:2d}       {line[0]:>13.8f} {line[1]:>13.8f} {line[2]:>13.8f}\n"
+                text += f"{Z:>2}       {line[0]:>13.8f} {line[1]:>13.8f} {line[2]:>13.8f}\n"
+            else:
+                text += f"{Z:2d}       {line[0]:>13.8f} {line[1]:>13.8f} {line[2]:>13.8f}\n"
 
         text += "\n"
         if footer is not None:
@@ -150,7 +156,7 @@ class GaussianFile(File):
         Args:
             filename (str): path to the new file
             molecule (int): which molecule to use -- passed to ``self.get_molecule()``.
-                Default is -1 (e.g. the last molecule), but positive integers will select from self.molecules (1-indexed).
+                Default is -1 (e.g. the last molecule), but positive integers will select from self.ensemble(1-indexed).
                 A ``Molecule`` object can also be passed, in which case that molecule will be written to the file.
             route_card (str): route card for new file
             link0 (dict): dictionary of Link 0 commands (e.g. {"mem": "32GB", "nprocshared": 16}
@@ -180,10 +186,14 @@ class GaussianFile(File):
         """
         Returns the imaginary frequencies, rounded to the nearest integer.
         """
-        if JobType.FREQ in self.job_types:
-            return list(map(int, np.array(self.frequencies)[np.array(self.frequencies) < 0]))
+        if (JobType.FREQ in self.job_types) and (self.ensemble[-1:,"frequencies"] is not None):
+            freqs = self.ensemble[-1:,"frequencies"]
+            if not isinstance(freqs, list) or len(freqs) == 0:
+                return list()
+            else:
+                return list(map(int, np.array(freqs)[np.array(freqs) < 0]))
         else:
-            raise TypeError("not a frequency job! can't get # imaginary frequencies!")
+            return list()
 
     @classmethod
     def read_file(cls, filename, return_lines=False):
@@ -191,6 +201,8 @@ class GaussianFile(File):
         Reads a Gaussian``.out`` or ``.gjf`` file and populates the attributes accordingly.
         Only footers from ``opt=modredundant`` can be read automatically --  ``genecep`` custom basis sets, &c must be specified manually.
 
+        Note:
+
         Will throw ``ValueError`` if there have been no successful iterations.
 
         Args:
@@ -200,14 +212,16 @@ class GaussianFile(File):
             ``GaussianFile`` object (or list of ``GaussianFile`` objects for Link1 files)
             (optional) the lines of the file (or list of lines of file for Link1 files)
         """
-        if re.search("gjf$", filename):
+        if re.search("gjf$", filename) or re.search("com$", filename):
             return cls._read_gjf_file(filename, return_lines)
 
         link1_lines = parse.split_link1(super().read_file(filename))
         files = []
-        for lines in link1_lines:
+        for link1idx, lines in enumerate(link1_lines):
             #### automatically assign job types based on header
             header = parse.search_for_block(lines, "#p", "----")
+            if header is None:
+                raise ValueError("can't find route card! (perhaps '#p' wasn't employed?)")
             job_types = cls._assign_job_types(header)
 
             link0 = parse.extract_link0(lines)
@@ -234,32 +248,39 @@ class GaussianFile(File):
                 footer = "\n".join([" ".join(list(filter(None, line.split(" ")))) for line in footer.split("\n")])
 
             bonds = parse.read_bonds(lines)
-            charge = parse.find_parameter(lines, "Multiplicity", expected_length=6, which_field=2)[0]
-            multip = parse.find_parameter(lines, "Multiplicity", expected_length=6, which_field=5)[0]
+            charge = parse.find_parameter(lines, "Multiplicity", expected_length=4, which_field=1, split_on="=")[0]
+            multip = parse.find_parameter(lines, "Multiplicity", expected_length=4, which_field=3, split_on="=")[0]
 
-            f = GaussianFile(atomic_numbers, geometries, bonds, job_types=job_types, charge=charge, multiplicity=multip)
-            f.energies = energies
-            f.scf_iterations = scf_iterations
-            f.route_card = header
-            f.link0 = link0
-            f.footer = footer
-            f.success = success
+            f = GaussianFile(job_types=job_types, route_card=header, link0=link0, footer=footer, success=success)
+
+            molecules = [None] * len(geometries)
+            properties = [{} for _ in range(len(geometries))]
+            for idx, geom in enumerate(geometries):
+                molecules[idx] = Molecule(atomic_numbers, geom, charge=charge, multiplicity=multip, bonds=bonds)
+                properties[idx]["energy"] = energies[idx]
+                properties[idx]["scf_iterations"] = scf_iterations[idx]
+                properties[idx]["link1_idx"] = link1idx
+                properties[idx]["filename"] = filename
 
             #### now for some job-type specific attributes
             if JobType.OPT in job_types:
-                f.rms_forces = parse.find_parameter(lines, "RMS\s+Force", expected_length=5, which_field=2)
-                f.rms_displacements = parse.find_parameter(lines, "RMS\s+Displacement", expected_length=5, which_field=2)
+                rms_forces = parse.find_parameter(lines, "RMS\s+Force", expected_length=5, which_field=2)
+                rms_displacements = parse.find_parameter(lines, "RMS\s+Displacement", expected_length=5, which_field=2)
+
+                for idx, force in enumerate(rms_forces):
+                    properties[idx]["rms_force"] = force
+                    properties[idx]["rms_displacement"] = rms_displacements[idx]
 
             if JobType.FREQ in job_types:
                 enthalpies = parse.find_parameter(lines, "thermal Enthalpies", expected_length=7, which_field=6)
                 if len(enthalpies) == 1:
-                    f.enthalpy = enthalpies[0]
+                    properties[-1]["enthalpy"] = enthalpies[0]
                 elif len(enthalpies) > 1:
                     raise ValueError("too many enthalpies found!")
 
                 gibbs_vals = parse.find_parameter(lines, "thermal Free Energies", expected_length=8, which_field=7)
                 if len(gibbs_vals) == 1:
-                    f.gibbs_free_energy = gibbs_vals[0]
+                    properties[-1]["gibbs_free_energy"] = gibbs_vals[0]
                 elif len(gibbs_vals) > 1:
                     raise ValueError("too many gibbs free energies found!")
 
@@ -268,15 +289,42 @@ class GaussianFile(File):
                     frequencies += parse.find_parameter(lines, "Frequencies", expected_length=5, which_field=2)
                     frequencies += parse.find_parameter(lines, "Frequencies", expected_length=5, which_field=3)
                     frequencies += parse.find_parameter(lines, "Frequencies", expected_length=5, which_field=4)
-                    f.frequencies = sorted(frequencies)
+                    properties[-1]["frequencies"] = sorted(frequencies)
                 except:
                     raise ValueError("error finding frequencies")
 
             if JobType.NMR in job_types:
-                assert len(f.molecules) == 1, "NMR jobs should not be combined with optimizations!"
-                nmr_shifts = parse.read_nmr_shifts(lines, f.molecules[-1].num_atoms())
-                f.molecules[0].nmr_isotropic = nmr_shifts.view(OneIndexedArray)
+                assert len(molecules) == 1, "NMR jobs should not be combined with optimizations!"
+                nmr_shifts = parse.read_nmr_shifts(lines, molecules[0].num_atoms())
+                properties[0]["isotropic_shielding"] = nmr_shifts.view(OneIndexedArray)
 
+            if JobType.FORCE in job_types:
+                assert len(molecules) == 1, "force jobs should not be combined with optimizations!"
+                forces = parse.read_forces(lines)
+                properties[0]["forces"] = forces
+
+            if JobType.POP in job_types:
+                if re.search("hirshfeld", f.route_card):
+                    charges, spins = parse.read_hirshfeld_charges(lines)
+                    properties[-1]["hirshfeld_charges"] = charges
+                    properties[-1]["hirshfeld_spins"] = spins
+
+            try:
+                charges = parse.read_mulliken_charges(lines)
+                properties[-1]["mulliken_charges"] = charges
+            except:
+                pass
+
+            try:
+                dipole = parse.read_dipole_moment(lines)
+                properties[-1]["dipole_moment"] = dipole
+            except:
+                pass
+
+            for mol, prop in zip(molecules, properties):
+                f.ensemble.add_molecule(mol, properties=prop)
+
+            f.check_has_properties()
             files.append(f)
 
         if return_lines:
@@ -293,7 +341,7 @@ class GaussianFile(File):
     @classmethod
     def _read_gjf_file(cls, filename, return_lines=False):
         """
-        Reads a Gaussian ``.gjf`` file and populates the attributes accordingly.
+        Reads a Gaussian ``.gjf`` or ``.com`` file and populates the attributes accordingly.
 
         Args:
             filename (str): path to the out file
@@ -312,7 +360,7 @@ class GaussianFile(File):
         multip = None
         in_geom = False
         atomic_numbers = []
-        geometries = []
+        geometry = []
 
         for idx, line in enumerate(lines):
             if header is None:
@@ -353,9 +401,9 @@ class GaussianFile(File):
                     assert len(pieces) == 4, f"can't parse line {line}"
 
                     atomic_numbers.append(pieces[0])
-                    geometries.append([pieces[1], pieces[2], pieces[3]])
+                    geometry.append([pieces[1], pieces[2], pieces[3]])
 
-            if (in_geom == False) and (len(geometries) > 0):
+            if (in_geom == False) and (len(geometry) > 0):
                 if footer:
                     footer = footer + "\n" + line
                 else:
@@ -367,16 +415,10 @@ class GaussianFile(File):
         except:
             atomic_numbers = np.array(list(map(get_number, atomic_numbers)), dtype=np.int8)
 
-        geometries = np.array([geometries])
         job_types = cls._assign_job_types(header)
 
-        f = GaussianFile(atomic_numbers, geometries, job_types=job_types, charge=charge, multiplicity=multip)
-        f.route_card = header
-        f.link0 = link0
-        f.footer = footer
-        f.title = title
-        f.success = 0
-
+        f = GaussianFile(job_types=job_types, route_card=header, link0=link0, footer=footer, title=title)
+        f.ensemble.add_molecule(Molecule(atomic_numbers, geometry, charge=charge, multiplicity=multip))
         if return_lines:
             return f, lines
         else:
@@ -386,7 +428,7 @@ class GaussianFile(File):
         """
         Returns the last molecule (from an optimization job) or the only molecule (from other jobs).
 
-        If ``num`` is specified, returns ``self.molecules[num]``
+        If ``num`` is specified, returns ``self.ensemble.molecule_list()[num]``
         """
         # some methods pass num=None, which overrides setting the default above
         if num is None:
@@ -395,32 +437,14 @@ class GaussianFile(File):
         if not isinstance(num, int):
             raise TypeError("num must be int")
 
-        return self.molecules[num]
-
-    @classmethod
-    def write_ensemble_to_file(cls, filename, ensemble, route_cards, kwargs):
-        """
-        Writes an Ensemble to a file using Link1 specification.
-
-        Args:
-            filename (str): where to write the file
-            ensemble (Ensemble): ``Ensemble`` object to write
-            headers (list): headers for each ``write_molecule_to_file`` call
-            kwargs (list of dict): arguments for each ``write_molecule_to_file`` call
-        """
-        for idx, molecule in enumerate(ensemble.molecules):
-            if idx == 0:
-                cls.write_molecule_to_file(filename, molecule, route_cards[idx], append=False, **kwargs[idx])
-            else:
-                cls.write_molecule_to_file(filename, molecule, route_cards[idx], append=True, **kwargs[idx])
-
+        return self.ensemble.molecule_list()[num]
 
     @classmethod
     def _assign_job_types(cls, header):
         """
-        Assigns ``JobType`` objects from route card.
+        Assigns ``JobType`` objects from route card. ``Job.Type.SP`` is assigned by default.
 
-        For instance, "#p opt freq=noraman" would give an output of [JobType.OPT, JobType.FREQ].
+        For instance, "#p opt freq=noraman" would give an output of ``[JobType.SP, JobType.OPT, JobType.FREQ]``.
 
         Args:
             header (str): Gaussian route card
@@ -432,4 +456,98 @@ class GaussianFile(File):
         for name, member in JobType.__members__.items():
             if re.search(f" {member.value}", str(header), re.IGNORECASE):
                 job_types.append(member)
+        if JobType.SP not in job_types:
+            job_types.append(JobType.SP)
         return job_types
+
+    def check_has_properties(self):
+        """
+        Checks that the file has all the appropriate properties for its job types, and raises ValueError if not.
+
+        This only checks the last molecule in ``self.ensemble``, for now.
+        """
+        return True
+#        for job_type in self.job_types:
+#            for prop in EXPECTED_PROPERTIES[job_type.value]:
+#                if not self.ensemble.has_property(-1, prop):
+#                    raise ValueError(f"expected property {prop} for job type {job_type}, but it's not there!")
+
+    @classmethod
+    def write_ensemble_to_file(cls, filename, ensemble, route_card, link0={"mem": "32GB", "nprocshared": 16}, footer=None, title="title", print_symbol=False):
+            """
+            Write each structure in the specified ensemble to a single Gaussian input file
+            by using the Link1 specification.
+
+            Args:
+                filename (str): where to write the file
+                ensemble (Ensemble): ``Ensemble`` object to write
+                route_card (str or list): to use the same route card for every link, use a single string;
+                                          otherwise, provide a list whose entries parallel the ensemble members
+                link0 (dict or list of dicts): to use the same memory/processors for every link, use a single string;
+                                               otherwise, provide a list
+                footer (None/str or list): use None for no text after geometry, provide a str to specify a footer,
+                                           or provide some combination of the above as a list
+                title (str or list): use a single string to provide a generic title for every link or a list as above
+                print_symbol (bool or list): whether to print atomic symbols or atomic numbers in the geometry specification;
+                                             use a single bool or a list as above
+
+            """
+            n_geometries = len(ensemble)
+            assert len(ensemble) > 0, "cannot write a blank ensemble"
+
+            if isinstance(route_card, str):
+                assert re.match(r"^#p", route_card), "route card should start with #p: {route_card}"
+                route_card = [route_card for m in ensemble._items]
+            elif isinstance(route_card, list):
+                assert len(route_card) == n_geometries, f"expected {n_geometries} route cards but got {len(route_card)}"
+                for card in route_card:
+                    assert isinstance(card, str), "expected route card to be a str"
+                    assert re.match(r"^#p", route_card), "route card should start with #p: f{card}"
+            else:
+                raise ValueError(f"unexpected type for route_card: {str(type(route_card))}")
+
+            if isinstance(link0, dict):
+                link0 = [link0 for m in ensemble._items]
+            elif isinstance(link0, list):
+                assert len(link0) == n_geometries, f"expected {n_geometries} link0 entries, but got {len(link0)}"
+                for d in link0:
+                    assert isinstance(d, dict), f"expected dict for link0 but got {str(type(d))}"
+            else:
+                raise ValueError(f"unexpected type for link0: {str(type(link0))}")
+
+            if footer is None or isinstance(footer, str):
+                footer = [footer for m in ensemble._items]
+            elif isinstance(footer, list):
+                assert len(footer) == n_geometries, f"expected {n_geometries} footers, but got {len(footer)}"
+                for f in footer:
+                    assert f is None or isinstance(f, str), f"expected str or None for footer but got {str(type(f))}"
+            else:
+                raise ValueError(f"unexpected type for footer: {str(type(footer))}")
+
+            if isinstance(title, str):
+                assert len(title.strip()) > 0, "zero-length titles not allowed"
+                title = [title for m in ensemble._items]
+            elif isinstance(title, list):
+                assert len(title) == n_geometries, f"expected {n_geometries} route cards but got {len(title)}"
+                for card in title:
+                    assert isinstance(card, str), "expected title to be a str"
+                    assert len(title.strip()) > 0, "zero-length titles are not allowed"
+            else:
+                raise ValueError(f"unexpected type for title: {str(type(title))}")
+
+            if isinstance(print_symbol, bool):
+                print_symbol = [print_symbol for m in ensemble._items]
+            elif isinstance(print_symbol, list):
+                assert len(print_symbol) == n_geometries, f"expected {n_geometries} print_symbol entries but got {len(print_symbol)}"
+                for s in print_symbol:
+                    assert isinstance(s, bool), f"expected bool for print_symbol but got {str(type(s))}"
+            else:
+                raise ValueError(f"unexpected type for print_symbol: {str(type(print_symbol))}")
+
+            for idx, molecule in enumerate(ensemble._items):
+                if idx == 0:
+                    cls.write_molecule_to_file(filename, molecule, route_card[idx], link0[idx], footer=footer[idx], title=title[idx], print_symbol=print_symbol[idx], append=False)
+                else:
+                    cls.write_molecule_to_file(filename, molecule, route_card[idx], link0[idx], footer=footer[idx], title=title[idx], print_symbol=print_symbol[idx], append=True)
+
+
