@@ -4,7 +4,7 @@ import networkx as nx
 
 from abc import abstractmethod
 
-from cctk import File, Ensemble, ConformationalEnsemble
+from cctk import File, Ensemble, ConformationalEnsemble, Molecule
 from cctk.helper_functions import get_symbol, get_number
 
 
@@ -14,7 +14,7 @@ class MAEFile(File):
 
     Attributes:
         name (str): name of file
-        molecules (Ensemble): ``Ensemble`` or ``ConformationalEnsemble`` object
+        ensemble (Ensemble): ``Ensemble`` or ``ConformationalEnsemble`` object
     """
 
     def __init__(self, name=None):
@@ -40,11 +40,14 @@ class MAEFile(File):
 
         (geometries, symbols, bonds, p_names, p_vals, conformers) = cls._read_mae(filename, **kwargs)
         atomic_numbers = np.array([get_number(z) for z in symbols], dtype=np.int8)
+
         if conformers == True:
-            file.molecules = ConformationalEnsemble(geometries=geometries, atomic_numbers=atomic_numbers, bonds=bonds.edges())
+            file.ensemble = ConformationalEnsemble()
         else:
-            atomic_numbers = np.tile(atomic_numbers, (len(geometries), 1))
-            file.molecules = Ensemble(geometries=geometries, atomic_numbers=atomic_numbers, bonds=[b.edges() for b in bonds])
+            file.ensemble = Ensemble()
+
+        for geom in geometries:
+            file.ensemble.add_molecule(Molecule(atomic_numbers, geom, bonds=bonds.edges))
 
         return file, p_names, p_vals
 
@@ -265,7 +268,7 @@ class MAEFile(File):
         """
         Returns the last molecule from the ensemble.
 
-        If ``num`` is specified, returns ``self.molecules[num]``
+        If ``num`` is specified, returns ``self.ensemble.molecules[num]``
         """
         # some methods pass num=None, which overrides setting the default above
         if num is None:
@@ -274,4 +277,4 @@ class MAEFile(File):
         if not isinstance(num, int):
             raise TypeError("num must be int")
 
-        return self.molecules[num]
+        return self.ensemble.molecules[num]
