@@ -14,7 +14,7 @@ class MOL2File(File):
 
     Attributes:
         name (str): name of file
-        molecules (Ensemble): ``Ensemble`` or ``ConformationalEnsemble`` object
+        ensemble (Ensemble): ``Ensemble`` or ``ConformationalEnsemble`` object
     """
 
     def __init__(self, name=None):
@@ -38,11 +38,14 @@ class MOL2File(File):
 
         (geometries, symbols, atom_types, bonds, conformers) = cls._read_mol2(filename, **kwargs)
         atomic_numbers = np.array([get_number(z) for z in symbols], dtype=np.int8)
+
         if conformers == True:
-            file.molecules = ConformationalEnsemble(geometries=geometries, atomic_numbers=atomic_numbers, bonds=bonds.edges())
+            file.ensemble = ConformationalEnsemble()
         else:
-            atomic_numbers = np.tile(atomic_numbers, (len(geometries), 1))
-            file.molecules = Ensemble(geometries=geometries, atomic_numbers=atomic_numbers, bonds=[b.edges() for b in bonds])
+            file.ensemble = Ensemble()
+
+        for geom in geometries:
+            file.ensemble.add_molecule(Molecule(atomic_numbers, geom, bonds=bonds.edges))
 
         return file
 
@@ -258,7 +261,7 @@ class MOL2File(File):
         """
         Returns the last molecule from the ensemble.
 
-        If ``num`` is specified, returns ``self.molecules[num]``
+        If ``num`` is specified, returns ``self.ensemble.molecules[num]``
         """
         # some methods pass num=None, which overrides setting the default above
         if num is None:
@@ -267,7 +270,7 @@ class MOL2File(File):
         if not isinstance(num, int):
             raise TypeError("num must be int")
 
-        return self.molecules[num]
+        return self.ensemble.molecules[num]
 
     @classmethod
     def write_molecule_to_file(cls, filename, molecule, title=None):
@@ -295,17 +298,44 @@ class MOL2File(File):
 
         super().write_file(filename, text)
 
-    def write_file(self, filename, molecule=None, **kwargs):
+    def write_file(self, filename, molecule=-1, **kwargs):
         """
         Write a ``.mol2`` file, using object attributes.
 
         Args:
             filename (str): path to the new file
             molecule (int): which molecule to use -- passed to ``self.get_molecule()``.
-                Default is -1 (e.g. the last molecule), but positive integers will select from self.molecules (1-indexed).
+                Default is -1 (e.g. the last molecule), but positive integers will select from self.ensemble.molecules (0-indexed).
                 A ``Molecule`` object can also be passed, in which case that molecule will be written to the file.
         """
-        if not isinstance(molecule, Molecule):
-            molecule = self.get_molecule(molecule)
-
+        if molecule is None or isinstance(molecule, (np.integer, int)):
+            molecule = self.ensemble.molecules[molecule]
         self.write_molecule_to_file(filename, molecule, **kwargs)
+
+    @classmethod
+    def write_ensemble_to_file(cls, filename, ensemble):
+        """
+        Write each structure in the specified ensemble to a single mol2 file.
+
+        Args:
+            filename (str): where to write the file
+            ensemble (Ensemble): ``Ensemble`` object to write
+        """
+        text = ""
+        for molecule in ensemble.molecules:
+            text += f"# {ensemble.name}\n#\n#\n\n#\n#\n\n"
+            text += f"@<TRIPOS>MOLECULE\nMolecule Name\n{molecule.num_atoms()} {molecule.bonds.number_of_edges()}\nSMALL\nNO_CHARGES\n\n\n"
+            text += "@<TRIPOS>ATOM\n"
+            for idx, z in enumerate(molecule.atomic_numbers, start=1):
+                v = molecule.get_vector(idx)
+                text += f"{idx} {get_symbol(z)}{idx}    {v[0]: .4f}    {v[1]: .4f}    {v[2]: .4f} {get_symbol(z)}\n"
+            text += "@<TRIPOS>BOND\n"
+            count = 1
+            for atom1, atom2, weight in molecule.bonds.edges.data("weight", default=1):
+                text += f"{count} {atom1} {atom2} {weight}\n"
+                count += 1
+
+        super().write_file(filename, text)
+
+
+
