@@ -1,8 +1,7 @@
 import sys, re, math
 import numpy as np
 import networkx as nx
-
-from functools import lru_cache
+import scipy
 
 from cctk import OneIndexedArray
 from cctk.helper_functions import (
@@ -33,7 +32,6 @@ class Molecule:
         bonds (nx.Graph): Graph object containing connectivity information (1-indexed)
         charge (int): the charge of the molecule
         multiplicity (int): the spin state of the molecule (1 corresponds to singlet, 2 to doublet, 3 to triplet, etc. -- so a multiplicity of 1 is equivalent to S=0)
-        nmr_isotropic (cctk.OneIndexedArray): NMR isotropic shifts
     """
 
     def __init__(self, atomic_numbers, geometry, name=None, bonds=None, charge=0, multiplicity=1):
@@ -102,6 +100,12 @@ class Molecule:
             for bond in bonds:
                 self.add_bond(bond[0], bond[1])
 
+    def __str__(self):
+        if self.name is not None:
+            return f"Molecule (name={self.name}, {len(self.atomic_numbers)} atoms)"
+        else:
+            return f"Molecule ({len(self.atomic_numbers)} atoms)"
+
     def assign_connectivity(self, cutoff=0.5):
         """
         Automatically recalculates bonds based on covalent radii. If two atoms are closer than the sum of their covalent radii + 0.5 Angstroms, then they are considered bonded.
@@ -127,18 +131,19 @@ class Molecule:
 
         return self
 
-    def check_for_conflicts(self, min_buffer=-1, group1=None, group2=None):
+    def check_for_conflicts(self, min_buffer=1, group1=None, group2=None):
         """
         Automatically checks for conflicts based on covalent radii. If two atoms are closer than the sum of their covalent radii + buffer, then they are considered clashing.
         If `group1` and `group2` are selected, then conflicts will only be evaluated between these two groups of atoms.
 
         Args:
-            min_buffer (float): the threshold (in Angstroms) for how close two covalent radii must be to be considered clashing. -1.0 A is default, for no particular reason.
+            min_buffer (float): the threshold (in Angstroms) for how close two covalent radii must be to be considered clashing. 1.0 A is default, empirically.
             group1 (list): atoms to evaluate against `group2` (if `None`, defaults to all atoms)
             group2 (list): atoms to evaluate against `group1` (if `None`, defaults to all atoms)
 
         Returns:
-            True if there are no conflicts, False (+ error) if there are
+            True if there are no conflicts
+            ValueError if there is a conflict
         """
 
         if group1 is None:
@@ -159,9 +164,8 @@ class Molecule:
                 r_j = get_covalent_radius(self.get_atomic_number(j))
 
                 # 0.5 A distance is used by RasMol and Chime (documentation available online) and works well, empirically
-                if distance < (r_i + r_j + min_buffer):
+                if distance < (r_i + r_j - min_buffer):
                     raise ValueError(f"atoms {i} and {j} are too close - distance {distance} A!")
-                    return False
 
         return True
 
@@ -254,9 +258,7 @@ class Molecule:
 
             return formula
 
-    #### very fast but causes errors sometimes... so i'm commenting this out until further consultation.
-    #    @lru_cache(maxsize=32)
-    def _get_bond_fragments(self, atom1, atom2, bond_order=1):
+    def _get_bond_fragments(self, atom1, atom2):
         """
         Returns the pieces of a molecule that one would obtain by breaking the bond between two atoms. Will throw ``ValueError`` if the atoms are in a ring.
         Useful for distance/angle/dihedral scans -- one fragment can be moved and the other held constant.
@@ -264,7 +266,6 @@ class Molecule:
         Args:
             atom1 (int): the number of the first atom
             atom2 (int): the number of the second atom
-            bond_order (int): bond order of bond between atom1 and atom2
 
         Returns:
             fragment1: the list of atoms in fragment 1 (containing atom1)
@@ -275,9 +276,7 @@ class Molecule:
         self._check_atom_number(atom1)
         self._check_atom_number(atom2)
 
-        if (not isinstance(bond_order, int)) or (bond_order < 0):
-            raise ValueError("invalid bond order!")
-
+        bond_order = self.get_bond_order(atom1, atom2)
         if self.bonds.has_edge(atom1, atom2):
             self.bonds.remove_edge(atom1, atom2)
 
@@ -770,14 +769,47 @@ class Molecule:
         Get the atomic number for a given atom.
 
         Args:
-            atom1 (int): number of the first atom
+            atom (int): number of the first atom
 
         Returns:
-            the atomic number of that atom
+            atomic_number (int): the atomic number of that atom
         """
         self._check_atom_number(atom)
         return self.atomic_numbers[atom]
 
+    def get_atomic_symbol(self, atom):
+        """
+        Get the atomic symbol for a given atom.
+
+        Args:
+            atom (int): number of the first atom
+
+        Returns:
+            atomic_symbol (str): the atomic symbol of that atom
+         """
+        atomic_number = self.get_atomic_number(atom)
+        return get_symbol(atomic_number)
+
+    def get_atomic_symbols(self):
+        """
+        Get a list of atomic symbols for this Molecule.
+
+        Returns:
+            atomic_symbols (cctk.OneIndexedArray): the atomic symbols
+        """
+        n_atoms = self.get_n_atoms()
+        l = [ self.get_atomic_symbol(i) for i in range(1,n_atoms+1) ]
+        return OneIndexedArray(l)
+
+    def get_n_atoms(self):
+        """
+        Determine how many atoms are in this Molecule.
+
+        Returns
+            n_atoms (int): the number of atoms
+        """
+        return len(self.atomic_numbers)
+
     def get_vector(self, atom, atom2=None, check=True):
         """
         Get the geometry vector for a given atom. If two atoms are specified, gives the vector connecting them (from ``atom2`` to ``atom``).
@@ -1114,3 +1146,10 @@ class Molecule:
         multiplicity = (s1+s2) * 2 + 1
 
         return Molecule(atoms, geoms, charge=charge, multiplicity=multiplicity)
+
+    def volume(self):
+        """
+        Returns volume calculated from ``scipy.spatial.ConvexHull``. Not a perfect approximation (assumes all atoms are points).
+        """
+        hull = scipy.spatial.ConvexHull(self.geometry.view(np.ndarray))
+        return hull.volume
