@@ -2,6 +2,7 @@ import numpy as np
 import re
 
 from cctk.helper_functions import get_symbol
+from cctk import OneIndexedArray
 
 """
 Functions to help with parsing Gaussian files
@@ -217,7 +218,7 @@ def read_bonds(lines):
         return None
 
 
-def find_parameter(lines, parameter, expected_length, which_field):
+def find_parameter(lines, parameter, expected_length, which_field, split_on=None):
     """
     Helper method to search through the output file and find key forces and displacements.
 
@@ -226,6 +227,7 @@ def find_parameter(lines, parameter, expected_length, which_field):
         parameter (string): test to search for
         expected_length (int): how many fields there should be
         which_field (int): which field the parameter is (zero-indexed)
+        split_on (str): additional non-space field to split
     Returns:
         a list of all the extracted values
     """
@@ -248,6 +250,11 @@ def find_parameter(lines, parameter, expected_length, which_field):
         for line in lines:
             if pattern.search(line):
                 fields = re.split(" +", line)
+                if split_on:
+                    fields2 = []
+                    for field in fields:
+                        fields2 = fields2 + field.split(split_on)
+                    fields = fields2
                 fields = list(filter(None, fields))
 
                 if len(fields) == expected_length:
@@ -295,7 +302,8 @@ def read_nmr_shifts(lines, num_atoms):
     # assumes that lines only come from one Link1 section
     shieldings = []
     for line in lines:
-        if line[14:23] == "Isotropic":
+        fields = line.split()
+        if len(fields) == 8 and fields[2] == "Isotropic" and fields[5] == "Anisotropy":
             fields = line.split()
             assert len(fields) == 8, f"Expected 8 fields on an NMR shielding output line but found {len(fields)} instead!"
             try:
@@ -360,3 +368,87 @@ def extract_link0(lines):
             output[pieces[0]] = pieces[1]
 
     return output
+
+def read_forces(lines):
+    """
+    Reads forces from a Gaussian ``force`` job.
+
+    Args:
+        lines (list): list of lines in file
+
+    Returns:
+        (n x 3) ``cctk.OneIndexedArray`` of forces
+    """
+    forces = []
+    force_block = search_for_block(lines, "Forces \(Hartrees/Bohr\)", "Cartesian Forces", join="\n")
+    for line in force_block.split("\n")[2:]:
+        fields = re.split(" +", line)
+        fields = list(filter(None, fields))
+
+        if len(fields) == 5:
+            forces.append([float(fields[2]), float(fields[3]), float(fields[4])])
+
+    return OneIndexedArray(forces)
+
+def read_mulliken_charges(lines):
+    """
+    Reads charges from a Gaussian ``pop`` job.
+
+    Args:
+        lines (list): list of lines in file
+
+    Returns:
+        ``cctk.OneIndexedArray`` of charges
+    """
+    charges = []
+    charge_block = search_for_block(lines, " Mulliken charges:", " Sum of Mulliken charges", join="\n")
+    for line in charge_block.split("\n")[2:]:
+        fields = re.split(" +", line)
+        fields = list(filter(None, fields))
+
+        if len(fields) == 3:
+            charges.append(float(fields[2]))
+
+    return OneIndexedArray(charges)
+
+def read_hirshfeld_charges(lines):
+    """
+    Reads charges from a Gaussian ``pop`` job.
+
+    Args:
+        lines (list): list of lines in file
+
+    Returns:
+        ``cctk.OneIndexedArray`` of charges
+        ``cctk.OneIndexedArray`` of spin densities
+    """
+    charges = []
+    spins = []
+    charge_block = search_for_block(lines, "Hirshfeld charges, spin densities, dipoles, and CM5 charges", " Hirshfeld charges", join="\n")
+    for line in charge_block.split("\n")[2:]:
+        fields = re.split(" +", line)
+        fields = list(filter(None, fields))
+
+        if len(fields) == 8:
+            charges.append(float(fields[2]))
+            spins.append(float(fields[3]))
+
+    return OneIndexedArray(charges), OneIndexedArray(spins)
+
+def read_dipole_moment(lines):
+    """
+    Reads dipole moment from a Gaussian job.
+
+    Args:
+        lines (list): list of lines in file
+
+    Returns:
+        dipole moment (magnitude only)
+    """
+    dipole_block = search_for_block(lines, "Dipole moment \(field-independent basis, Debye\)", "Quadrupole moment \(field-independent basis, Debye-Ang\):", join="\n")
+    for line in dipole_block.split("\n")[1:]:
+        fields = re.split(" +", line)
+        fields = list(filter(None, fields))
+
+        if len(fields) == 8:
+            return float(fields[7])
