@@ -52,6 +52,12 @@ class TestMolecule(unittest.TestCase):
         self.assertEqual(int(round(mol.get_dihedral(16,14,17,18))), 11)
         self.assertEqual(int(round(mol.get_dihedral(31,28,1,2))), 148)
 
+        atom_tuple = (1, 3, 5, 7)
+        mol.set_dihedral(*atom_tuple, 120)
+
+        atom_list = list(atom_tuple)
+        mol.set_dihedral(*atom_list, 120)
+
         theta = [1, 20, 89, 66, 180, 215, 333]
         for t in theta:
             mol.set_dihedral(1, 3, 5, 7, t)
@@ -123,5 +129,9 @@ class TestMolecule(unittest.TestCase):
         self.assertEqual(m3.charge, 1)
         self.assertEqual(m3.multiplicity, 1)
 
+    def test_volume(self):
+        mol = self.load_molecule()
+        self.assertEqual(mol.volume(), 80.42662712363737)
+
 if __name__ == '__main__':
     unittest.main()
